/*******************************************************************************
* File Name: IDAC_TEST.c  
* Version 1.60
*
*  Description:
*    This file provides the source code to the API for the 8-bit Current 
*    DAC (IDAC8) User Module.
*
*   Note:
*     
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "IDAC_TEST.h"   

uint8 IDAC_TEST_initVar = 0u;


/*******************************************************************************
* Function Name: IDAC_TEST_Init
********************************************************************************
* Summary:
*  Initialize to the schematic state.
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
*******************************************************************************/
void IDAC_TEST_Init(void) 
{
    IDAC_TEST_CR0 = (IDAC_TEST_MODE_I | IDAC_TEST_DEFAULT_RANGE )  ;    

    /* Set default data source */
    if(IDAC_TEST_DEFAULT_DATA_SRC != 0u )    
    {
        IDAC_TEST_CR1 = (IDAC_TEST_DEFAULT_CNTL | IDAC_TEST_DACBUS_ENABLE | IDAC_TEST_DEFAULT_POLARITY) ;   
    }
    else
    {
        IDAC_TEST_CR1 = (IDAC_TEST_DEFAULT_CNTL | IDAC_TEST_DACBUS_DISABLE | IDAC_TEST_DEFAULT_POLARITY) ;   
    } 
    
    /* Set default strobe mode */
    if(IDAC_TEST_DEFAULT_STRB != 0u)
    {
        IDAC_TEST_Strobe |= IDAC_TEST_STRB_EN ;
    }
    
    /* Set default speed */
    IDAC_TEST_SetSpeed(IDAC_TEST_DEFAULT_SPEED); 
    
    /* Set proper DAC trim */
    IDAC_TEST_DacTrim();   
}


/*******************************************************************************
* Function Name: IDAC_TEST_Enable
********************************************************************************
* Summary:
*     Enable the IDAC8
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
*******************************************************************************/
void IDAC_TEST_Enable(void) 
{
    IDAC_TEST_PWRMGR |= IDAC_TEST_ACT_PWR_EN;
    IDAC_TEST_STBY_PWRMGR |= IDAC_TEST_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: IDAC_TEST_Start
********************************************************************************
* Summary:
*  Set power level then turn on IDAC8.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Global variables:
*  IDAC_TEST_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void IDAC_TEST_Start(void) 
{
    /* Hardware initiazation only needs to occur the first time */
    if ( IDAC_TEST_initVar == 0u)  
    {     
        IDAC_TEST_Init();
       
        IDAC_TEST_initVar = 1;
    }  
   
    /* Enable power to DAC */
    IDAC_TEST_Enable();

    /* Set default value */
    IDAC_TEST_SetValue(IDAC_TEST_DEFAULT_DATA);

}


/*******************************************************************************
* Function Name: IDAC_TEST_Stop
********************************************************************************
* Summary:
*  Powers down IDAC8 to lowest power state.
*
* Parameters:  
*   (void)
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC_TEST_Stop(void) 
{
   /* Disble power to DAC */
   IDAC_TEST_PWRMGR &= ~IDAC_TEST_ACT_PWR_EN;
   IDAC_TEST_STBY_PWRMGR &= ~IDAC_TEST_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: IDAC_TEST_SetSpeed
********************************************************************************
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC_TEST_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   IDAC_TEST_CR0 &= ~IDAC_TEST_HS_MASK ;    
   IDAC_TEST_CR0 |=  ( speed & IDAC_TEST_HS_MASK) ;    
}


/*******************************************************************************
* Function Name: IDAC_TEST_SetPolarity
********************************************************************************
* Summary:
*  Sets IDAC to Sink or Source current
*
* Parameters:  
*  Polarity: Sets the IDAC to Sink or Source
*   0x00 - Source
*   0x04 - Sink
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC_TEST_SetPolarity(uint8 polarity) 
{
   	IDAC_TEST_CR1 &= ~IDAC_TEST_IDIR_MASK;                /* clear polarity bit */
    IDAC_TEST_CR1 |= (polarity & IDAC_TEST_IDIR_MASK);    /* set new value */
    
    IDAC_TEST_DacTrim();
}

/*******************************************************************************
* Function Name: IDAC_TEST_SetRange
********************************************************************************
* Summary:
*  Set current range
*
* Parameters:  
*  Range:  Sets on of four valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC_TEST_SetRange(uint8 range) 
{
   IDAC_TEST_CR0 &= ~IDAC_TEST_RANGE_MASK ;       /* Clear existing mode */
   IDAC_TEST_CR0 |= ( range & IDAC_TEST_RANGE_MASK );  /*  Set Range  */
   IDAC_TEST_DacTrim();
}


/*******************************************************************************
* Function Name: IDAC_TEST_SetValue
********************************************************************************
* Summary:
*  Set DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC_TEST_SetValue(uint8 value) 
{
   IDAC_TEST_Data = value;         /*  Set Value  */
   
   /* PSOC3 silicons earlier to ES3 require a double write */
   #if (IDAC_TEST_PSOC3_ES2 || IDAC_TEST_PSOC5_ES1)
       IDAC_TEST_Data = value;
   #endif
}

/*******************************************************************************
* Function Name: IDAC_TEST_DacTrim
********************************************************************************
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*  None
*
* Return: 
*  (void) 
*
* Theory: 
*  Trim values for the IDAC blocks are stored in the "Customer Table" area in 
*  Row 1 of the Hidden Flash.  There are 8 bytes of trim data for each IDAC block.
*  The values are:
*       I Gain offset, min range, Sourcing
*       I Gain offset, min range, Sinking
*       I Gain offset, med range, Sourcing
*       I Gain offset, med range, Sinking
*       I Gain offset, max range, Sourcing
*       I Gain offset, max range, Sinking
*       V Gain offset, 1V range
*       V Gain offset, 4V range
*
* Side Effects:
*
*******************************************************************************/
void IDAC_TEST_DacTrim(void) 
{
    uint8 mode;

    mode = ((IDAC_TEST_CR0 & IDAC_TEST_RANGE_MASK) >> 1);
    
    if((IDAC_TEST_IDIR_MASK & IDAC_TEST_CR1) == IDAC_TEST_IDIR_SINK)
    {
        mode++;
    }

    IDAC_TEST_TR = CY_GET_XTND_REG8((uint8 *)(IDAC_TEST_DAC_TRIM_BASE + mode));
}


/* [] END OF FILE */
