/*******************************************************************************
* File Name: LP_Filter.c
* Version 1.50
*
* Description:
*  This file provides the API source code for the FILT component.
*
* Note:
*  
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "LP_Filter_PVT.h"


/*******************************************************************************
* FILT component internal variables.
*******************************************************************************/

uint8 XDATA LP_Filter_initVar = 0u;


/*******************************************************************************
* Function Name: LP_Filter_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  LP_Filter_Start().
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrance:
*  Yes
*
*******************************************************************************/
void LP_Filter_Init(void) 
{
        /* Power on DFB before initializing the RAMs */
    LP_Filter_PM_ACT_CFG_REG |= LP_Filter_PM_ACT_MSK;

    /* Turn off Run Bit */
    LP_Filter_CR_REG &= ~LP_Filter_RUN_MASK;
                
    /* Enable the DFB RAMS */
    LP_Filter_RAM_EN_REG = LP_Filter_RAM_DIR_BUS;
        
    /* Put DFB RAM on the bus */
    LP_Filter_RAM_DIR_REG = LP_Filter_RAM_DIR_BUS;
        
    /* Write DFB RAMs */
    /* Control Store RAMs */
    cymemcpy( LP_Filter_CSA_RAM,
        LP_Filter_control, LP_Filter_CSA_RAM_SIZE); 
    cymemcpy(LP_Filter_CSB_RAM,
        LP_Filter_control, LP_Filter_CSB_RAM_SIZE); 
    /* CFSM RAM */
    cymemcpy(LP_Filter_CFSM_RAM,
        LP_Filter_cfsm, LP_Filter_CFSM_RAM_SIZE); 
    /* DAta RAMs */
    cymemcpy(LP_Filter_DA_RAM,
        LP_Filter_data_a, LP_Filter_DA_RAM_SIZE); 
    cymemcpy(LP_Filter_DB_RAM,
        LP_Filter_data_b, LP_Filter_DB_RAM_SIZE); 
    /* ACU RAM */
    cymemcpy(LP_Filter_ACU_RAM,
        LP_Filter_acu, LP_Filter_ACU_RAM_SIZE); 

    /* Take DFB RAM off the bus */
    LP_Filter_RAM_DIR_REG = LP_Filter_RAM_DIR_DFB;

    /* Set up interrupt and DMA events */
    LP_Filter_SetInterruptMode(LP_Filter_INIT_INTERRUPT_MODE);
    LP_Filter_SetDMAMode(LP_Filter_INIT_DMA_MODE);
        
    /* Clear any pending interrupts */
    LP_Filter_SR_REG = 0xff;
}


/*******************************************************************************
* Function Name: LP_Filter_Enable
********************************************************************************
*  
* Summary: 
*  Enables the DFB run bit.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrance:
*  Yes
*
*******************************************************************************/
void LP_Filter_Enable(void) 
{
    /* Power on DFB in Active mode */
    LP_Filter_PM_ACT_CFG_REG |= LP_Filter_PM_ACT_MSK;
        
    /* Power on DFB in Alternative Active mode */
    LP_Filter_PM_STBY_CFG_REG |= LP_Filter_PM_STBY_MSK;

    /* Turn on Run Bit */
    LP_Filter_CR_REG |= LP_Filter_RUN_MASK;
}


/*******************************************************************************
* Function Name: LP_Filter_Start
********************************************************************************
*
* Summary:
*  This method does the prep work necessary to setup DFB.  This includes loading 
*
* Parameters:  
*  void
* 
* Return: 
*  void
*
* Global variables:
*  LP_Filter_initVar:  Used to check the initial configuration,
*  modified when this function is called for the first time.
*
* Note: 
*  Use LP_Filter_InterruptConfig to control which events trigger 
*  interrupts in the DFB. 
*
* Reentrance:
*  No
*
*******************************************************************************/
void LP_Filter_Start()
{
     /* If not Initialized then initialize all required hardware and software */
    if(LP_Filter_initVar == 0u)
    {
        LP_Filter_Init();
        LP_Filter_initVar = 1u;
    }

    /* Enable the DFB block */
    LP_Filter_Enable();
}


/*******************************************************************************
* Function Name: LP_Filter_Stop
********************************************************************************
*
* Summary:
*  Turn off the run bit.  If DMA control is used to feed the channels, allow 
*  arguments to turn one of the TD channels off. 
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrance:
*  Yes
*
*******************************************************************************/
void LP_Filter_Stop() 
{
    /* Power off DFB in Active mode */
    LP_Filter_PM_ACT_CFG_REG &= ~LP_Filter_PM_ACT_MSK;
    
    /* Power off DFB in Alternative Active mode */
    LP_Filter_PM_STBY_CFG_REG &= ~LP_Filter_PM_STBY_MSK;
        
    LP_Filter_CR_REG &= ~(LP_Filter_RUN_MASK);
}


/*******************************************************************************
* Function Name: LP_Filter_Read8
********************************************************************************
*
* Summary:
*  Get the value in one of the DFB Output Holding Registers 
*
* Parameters:  
*  channel:  LP_Filter_CHANNEL_A or LP_Filter_CHANNEL_B
*            
* Return: 
*  The most significant 8 bits of the current output value sitting in the 
*  selected channel's holding register or 0x00 for invalid channel numbers.
*
* Reentrance:
*  Yes
*
*******************************************************************************/
uint8 LP_Filter_Read8(uint8 channel) 
{
    if (channel == LP_Filter_CHANNEL_A)
    {
        return LP_Filter_HOLDAH_REG;
    }
    else if (channel == LP_Filter_CHANNEL_B)
    {
        return LP_Filter_HOLDBH_REG;
    }
    else
    {
        return 0;
    }
}


/*******************************************************************************
* Function Name: LP_Filter_Read16
********************************************************************************
*
* Summary:
*  Get the value in one of the DFB Output Holding Registers 
*
* Parameters:  
*  channel:  LP_Filter_CHANNEL_A or LP_Filter_CHANNEL_B
*            
* Return: 
*  The most significant 16 bits of the current output value sitting in the 
*  selected channel's holding register or 0x0000 for invalid channel numbers
*
* Note:
*  Order of the read is important.  On the read of the high byte, the DFB clears
*  the data ready bit.
*
* Reentrance:
*  Yes
*
*******************************************************************************/
#if defined(__C51__) || defined(__CX51__) 

    uint16 LP_Filter_Read16(uint8 channel) 
    {
        uint16 val;
    
        if (channel == LP_Filter_CHANNEL_A)
        {        
            val = LP_Filter_HOLDAM_REG;
            val |= (uint16)(LP_Filter_HOLDAH_REG) << 8;
            return val;
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {      
            val = LP_Filter_HOLDBM_REG;
            val |= (uint16)LP_Filter_HOLDBH_REG << 8;
            return val;
        }
        else
        {
            return 0;
        }
    }

#else

    uint16 LP_Filter_Read16(uint8 channel) 
    {
        if (channel == LP_Filter_CHANNEL_A)
        {        
            return LP_Filter_HOLDA16_REG;
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {      
            return LP_Filter_HOLDB16_REG;
        }
        else
        {
            return 0;
        }
    }

#endif /* defined(__C51__) || defined(__CX51__) */


/*******************************************************************************
* Function Name: LP_Filter_Read24
********************************************************************************
*
* Summary:
*  Get the value in one of the DFB Output Holding Registers 
*
* Parameters:  
*  channel:  LP_Filter_CHANNEL_A or LP_Filter_CHANNEL_B
*            
* Return: 
*  The current 24-bit output value sitting in the selected channel's
*  holding register or 0x00000000 for invalid channel numbers
*
* Note:
*  Order of the read is important.  On the read of the high byte, the DFB clears
*  the data ready bit.
*
* Reentrance:
*  Yes
*
*******************************************************************************/
#if defined(__C51__) || defined(__CX51__)

    uint32 LP_Filter_Read24(uint8 channel) 
    {
        uint32 val;
    
        if (channel == LP_Filter_CHANNEL_A)
        {        
            val = LP_Filter_HOLDA_REG;
            val |= (uint32)(LP_Filter_HOLDAM_REG) << 8;
            val |= (uint32)(LP_Filter_HOLDAH_REG) << 16;
            
            /* SignExtend */
            if(val & LP_Filter_SIGN_BIT)
                val |= LP_Filter_SIGN_BYTE;
                
            return val;
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {      
            val = LP_Filter_HOLDB_REG;
            val |= (uint32)LP_Filter_HOLDBM_REG << 8;
            val |= (uint32)LP_Filter_HOLDBH_REG << 16;
            
            /* SignExtend */
            if(val & LP_Filter_SIGN_BIT)
                val |= LP_Filter_SIGN_BYTE;
            
            return val;
        }
        else
        {
            return 0;
        }
    }

#else

    uint32 LP_Filter_Read24(uint8 channel) 
    {
        if (channel == LP_Filter_CHANNEL_A)
        {        
            return LP_Filter_HOLDA24_REG;
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {      
            return LP_Filter_HOLDB24_REG;
        }
        else
        {
            return 0;
        }
    }

#endif /* defined(__C51__) || defined(__CX51__) */


/*******************************************************************************
* Function Name: LP_Filter_Write8
********************************************************************************
*
* Summary:
*  Set the value in one of the DFB Input Staging Registers 
*
* Parameters:  
*  channel:  Use either LP_Filter_CHANNEL_A or 
*            LP_Filter_CHANNEL_B as arguments to the function.  
*  sample:   The 8-bit, right justified input sample. 
*
* Return: 
*  void
*
* Note:
*  Order of the write is important.  On the load of the high byte, the DFB sets
*  the input ready bit.
*
* Reentrance:
*  Yes
*
*******************************************************************************/
void LP_Filter_Write8(uint8 channel, uint8 sample) 
{
    if (channel == LP_Filter_CHANNEL_A)
    {
        LP_Filter_STAGEAH_REG = sample;
    }
    else if (channel == LP_Filter_CHANNEL_B)
    {
        LP_Filter_STAGEBH_REG = sample;
    }
    /* No Final else statement: No value is loaded on bad channel input */
}


/*******************************************************************************
* Function Name: LP_Filter_Write16
********************************************************************************
*
* Summary:
*  Set the value in one of the DFB Input Staging Registers 
*
* Parameters:  
*  channel:  Use either LP_Filter_CHANNEL_A or 
*            LP_Filter_CHANNEL_B as arguments to the function.  
*  sample:   The 16-bit, right justified input sample. 
*
* Return: 
*  void
*
* Note:
*  Order of the write is important.  On the load of the high byte, the DFB sets
*  the input ready bit.
*
* Reentrance:
*  Yes
*
*******************************************************************************/
#if defined(__C51__) || defined(__CX51__)

    void LP_Filter_Write16(uint8 channel, uint16 sample) 
    {
        /* Write the STAGE MSB reg last as it signals a complete wrote to the DFB.*/
        if (channel == LP_Filter_CHANNEL_A)
        {
            LP_Filter_STAGEAM_REG = (uint8)(sample);
            LP_Filter_STAGEAH_REG = (uint8)(sample >> 8 );
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {
            LP_Filter_STAGEBM_REG = (uint8)(sample);
            LP_Filter_STAGEBH_REG = (uint8)(sample >> 8);
        }
        /* No Final else statement: No value is loaded on bad channel input */
    }

#else

    void LP_Filter_Write16(uint8 channel, uint16 sample) 
    {
        if (channel == LP_Filter_CHANNEL_A)
        {
            LP_Filter_STAGEA16_REG = sample;
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {
            LP_Filter_STAGEB16_REG = sample;
        }
        /* No Final else statement: No value is loaded on bad channel input */
    }

#endif /* defined(__C51__) || defined(__CX51__) */


/*******************************************************************************
* Function Name: LP_Filter_Write24
********************************************************************************
*
* Summary:
*  Set the value in one of the DFB Input Staging Registers 
*
* Parameters:  
*  channel:  Use either LP_Filter_CHANNEL_A or 
*            LP_Filter_CHANNEL_B as arguments to the function.  
*  sample:   The 24-bit, right justified input sample inside of a uint32. 
*
* Return: 
*  void
*
* Note:
*  Order of the write is important.  On the load of the high byte, the DFB sets
*  the input ready bit.
*
* Reentrance:
*  Yes
*
*******************************************************************************/
#if defined(__C51__) || defined(__CX51__)

    void LP_Filter_Write24(uint8 channel, uint32 sample) 
    {
        /* Write the STAGE LSB reg last as it signals a complete wrote to the DFB.*/
        if (channel == LP_Filter_CHANNEL_A)
        {
            LP_Filter_STAGEA_REG  = (uint8)(sample);
            LP_Filter_STAGEAM_REG = (uint8)(sample >> 8 );
            LP_Filter_STAGEAH_REG = (uint8)(sample >> 16);
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {
            LP_Filter_STAGEB_REG = (uint8)(sample);
            LP_Filter_STAGEBM_REG = (uint8)(sample >> 8);
            LP_Filter_STAGEBH_REG = (uint8)(sample >> 16);
        }
        /* No Final else statement: No value is loaded on bad channel input */
    }

#else

    void LP_Filter_Write24(uint8 channel, uint32 sample) 
    {
        if (channel == LP_Filter_CHANNEL_A)
        {
            LP_Filter_STAGEA24_REG = sample;
        }
        else if (channel == LP_Filter_CHANNEL_B)
        {
            LP_Filter_STAGEB24_REG = sample;
        }
        /* No Final else statement: No value is loaded on bad channel input */
    }

#endif /* defined(__C51__) || defined(__CX51__) */


/* [] END OF FILE */
